@@ -1,13 +1,11 @@
 class ClientError(Exception):
-    """ Error thrown by Schema Registry clients """
+    """Error thrown by Schema Registry client"""
 
-    def __init__(
-        self, message: str, http_code: int = None, server_traceback: str = None
-    ) -> None:
+    def __init__(self, message: str, http_code: int = None, server_traceback: str = None) -> None:
         self.message = message
         self.server_traceback = server_traceback
         self.http_code = http_code
-        super(ClientError, self).__init__(self.__str__())
+        super().__init__(message)
 
     def __repr__(self) -> str:
         return f"ClientError(error={self.message})"
